from pydantic import EmailStr
from pydantic.dataclasses import dataclass

from typing import Optional, List


@dataclass(unsafe_hash=True)
class Personalia:
    phone: str
    address: str
    nationality: str
    email: EmailStr


@dataclass
class Job:
    title: str
    industry: str
    dates: str
    description: Optional[str]


@dataclass
class Education:
    diploma: str
    institution: str
    dates: str
    description: str


@dataclass
class Activity:
    title: str
    organization: str
    dates: str
    description: str


@dataclass
class Project:
    title: str
    description: str


@dataclass
class Skill:
    category: str
    keywords: List[str]


@dataclass
class CV:
    name: str
    personalia: Personalia
    experience: List[Job]
    education: List[Education]
    skills: List[Skill]
    projects: List[Project]
    activities: List[Activity]



import datetime
import subprocess
from os import PathLike
from pathlib import Path

import click
import yaml
from cvgen.models import CV
from jinja2 import Environment, FileSystemLoader, select_autoescape
from pydantic import ValidationError
from weasyprint import CSS, HTML


CURRENT_DIR = Path(__file__).parent
jinja2_env = Environment(
    loader=FileSystemLoader(str(CURRENT_DIR / "templates")),
    autoescape=select_autoescape(["html"]),
)


def load_data(path: PathLike) -> CV:
    with open(path) as f:
        data = yaml.safe_load(f)
    try:
        data = CV(**data)
        return data
    except ValidationError as e:
        click.echo("Input data file invalid", err=True)
        click.echo(e, err=True)
        raise


def render(data: CV, to: PathLike):
    template = jinja2_env.get_template("index.html.j2")
    rendered = template.render(
        cv=data, timestamp=datetime.datetime.now().strftime("%d-%m-%Y")
    )
    HTML(string=rendered).write_pdf(
        to, stylesheets=[CSS(CURRENT_DIR / "templates" / "style.css")]
    )


@click.command()
@click.argument("input_data")
def main(input_data: PathLike):
    click.echo("CVGen started!")
    cv = load_data(input_data)
    click.echo("Data has been loaded successfully.")
    output_file = Path(f"{cv.name} CV @ {datetime.datetime.now():%d-%m-%Y}.pdf")

    click.echo(f'Rendering pdf to "{output_file}" ...')
    render(cv, output_file)
    click.echo("PDF has rendered.")
    subprocess.Popen(["open", output_file])



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['cvgen']

package_data = \
{'': ['*'], 'cvgen': ['templates/*']}

install_requires = \
['click>=7.0,<8.0',
 'email-validator>=1.0,<2.0',
 'jinja2>=2.10,<3.0',
 'pydantic>=0.25.0,<0.26.0',
 'pyyaml>=5.1,<6.0',
 'weasyprint>=47.0,<48.0']

entry_points = \
{'console_scripts': ['cvgen = cvgen.main:main']}

setup_kwargs = {
    'name': 'cvgen',
    'version': '0.1.4',
    'description': 'A tool for easily creating a good-looking CV in PDF format from .yaml data, HTML & CSS.',
    'long_description': None,
    'author': 'Vadim Galaktionov',
    'author_email': 'vadim@galaktionov.nl',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
